uniform lowp float amount;
uniform lowp float anglePower;
uniform lowp float radius;
uniform lowp float minAngle;
uniform lowp float maxAngle;
uniform lowp float rotationSpeed;

/////////////////////////////////////////////////////////
// BlackHole

//The current foreground texture co-ordinate
varying mediump vec2 vTex;
//The foreground texture sampler, to be sampled at vTex
uniform lowp sampler2D samplerFront;
//The current foreground rectangle being rendered
uniform mediump vec2 srcStart;
uniform mediump vec2 srcEnd;
//The current foreground source rectangle being rendered
uniform mediump vec2 srcOriginStart;
uniform mediump vec2 srcOriginEnd;
//The current foreground source rectangle being rendered, in layout 
uniform mediump vec2 layoutStart;
uniform mediump vec2 layoutEnd;
//The background texture sampler used for background - blending effects
uniform lowp sampler2D samplerBack;
//The current background rectangle being rendered to, in texture co-ordinates, for background-blending effects
uniform mediump vec2 destStart;
uniform mediump vec2 destEnd;
//The time in seconds since the runtime started. This can be used for animated effects
uniform mediump float seconds;
//The size of a texel in the foreground texture in texture co-ordinates
uniform mediump vec2 pixelSize;
//The current layer scale as a factor (i.e. 1 is unscaled)
uniform mediump float layerScale;
//The current layer angle in radians.
uniform mediump float layerAngle;

mediump vec4 blackhole(mediump vec2 vTex)
{
	// normalize from spritesheet
    mediump vec4 tcolor, texColor;
    mediump vec2 myAngle;
    mediump float dist;
    
    mediump vec2 vUv = (vTex-srcOriginStart)/(srcOriginEnd-srcOriginStart);
    mediump vec2 center = (srcOriginStart + srcOriginEnd) / 2.0;
    
    dist =  distance(vUv,center);
    
    myAngle = normalize(center - vUv) * (1.0-pow(dist, 1.0))/amount;
    
    vUv = vUv-myAngle;
    // denormalize for spritesheet
    vUv = vUv * (srcOriginEnd - srcOriginStart) + srcOriginStart;
    
    mediump vec2 n = (vUv - srcStart) / (srcEnd - srcStart);
    mediump vec2 nOriginal = (vTex - srcStart) / (srcEnd - srcStart);
    texColor = texture2D(samplerFront, vTex);
    tcolor = texture2D(samplerBack, mix(destStart, destEnd, mix(nOriginal, n, texColor.a)));
    return tcolor;
    
}

mediump float unlerp(mediump float min, mediump float max, mediump float value) {
  return (value - min) / (max - min);
}

void main(void)
{
	mediump vec2 center = vec2(0.5, 0.5);
	
	// convert back to [0, 1] range across source rectangle
	mediump vec2 srcOriginSize = srcOriginEnd - srcOriginStart;
	mediump vec2 tex = ((vTex - srcOriginStart) / srcOriginSize);
	
	mediump float dist = distance(center, tex);
	tex -= center;
	
	if (dist < radius)
	{
		mediump float angle = mix(maxAngle, minAngle, pow(unlerp(0.0, radius, dist), anglePower)) + seconds * rotationSpeed;
		mediump float percent = (radius - dist) / radius;
		mediump float theta = percent * percent * angle * 8.0;
		mediump float s = sin(theta);
		mediump float c = cos(theta);
		tex = vec2(dot(tex, vec2(c, -s)), dot(tex, vec2(s, c)));
	}
	
	tex += center;
	tex = clamp(tex, 0.0, 1.0);			// ensure no sampling outside source rect
	tex = (tex * srcOriginSize) + srcOriginStart;	// convert back relative to source rect

	gl_FragColor = blackhole(tex);
}